/******************************************************************************
 * This program reports the following parameters calculated from DDA:         *
 *   Cabs  - absorption cross section (m^2)			              *
 *   Csca  - scattering cross section (m^2)			              *
 *   Cbsc  - backscattering cross section (m^2)			              *
 *   g     - asymmetry parameter	(0 - 1)			              *
 *   alpha - calculated alpha statistic from Hanesch, 1999 (0-1)              *
 *           NOTE: dissertation location below                                * 
 *   arxz - aspect ratio calculated using a 2D image of X-Z axes  (0 - 1)     * 
 *   aryz - aspect ratio calculated using a 2D image of X-Z axes  (0 - 1)     *
 *           NOTE: calcuated using definition from Koroleve and Isaac, 2003   *
 *                 and Matrosov, 2007. Citations given below.                 *
 * For randomally oriented ice particles of the following shapes:             *
 *  SHAPE-ID	SHAPE#		SHAPE			                      * 
 *   ag2m        11     aggregate of 200 um, 6-bullet rosettes                *
 *   ag4m	 12	aggregate of 400 um, 6-bullet rosettes                *
 *   agc         13     aggregate of 200 and 400 um, 6-bullet rosettes        *
 * For frequency range: 10.65 - 183.31 GHz		                      *
 * For temperature : -10 C		                                      *
 *                                                                            *
 * References:							              *
 * Nowell, H. K., 2010: Modeling Snow Aggregates and the Single Scattering    *
 * Properties: Implications to Snowfall Remote Sensing. M.S. thesis, 68pp,    *  
 * Florida State University, Tallahassee.				      *
 *					                                      *
 * Citations:                                                                 *
 * Hanesch, M. (1999). Fall Velocity and Shape of Snowflakes, Ph.D.           *
 *      dissertation, 116 pp., Swiss Federal Institute of Technology          *
 *      Zurich,1999. Available for viewing at:                                *
 *      http://e-collection.library.ethz.ch/eserv/eth:23207/eth-23207-02.pdf  *
 * Korolev, A., & Isaac, G. (2003). Roundness and Aspect Ratio of Particles   *
 *      in Ice Clouds. Journal of the Atmospheric Sciences, 60(15), 1795-1808.* 
 * Matrosov, S. Y. (2007). Modeling Backscatter Properties of Snowfall at     *
 *      Millimeter Wavelengths. Journal of the Atmospheric Sciences, 64(5),   *
 *      1727-1736.                                                            *
 *                                                                            *
 * call by fortran:			                                      *
 *  call scatdb_ag(f,t,nshp,dmax,coabs,csca,cbsc,asym,re,alpha,arxz,aryz,  *
 *                iret) 						      *
 * call by c:							              *
 *  iret=scatdb_ag(f,t,nshape,dmax,&coabs,&csca,&cbsc,&asym,&re,&alpha,    *
 *              &arxz,&aryz,&is_loaded);    		                      *
 * Input:							              *
 *  f - frequency in GHz	real/float			              *
 *            Range: 10.65 - 183.31 GHz				              *
 *  t - temperature in K	real/float			              *
 *            Range: 263.0 K			      	                      *
 *  nshape - shape#	(see above)	integer/int		              *
 *            Range: 11-13					              *
 *  dmax - particle's maximum dimension (micron) real/float	              *
 *            Range is shape dependent as follows		              *
 *      SHAPE-ID	SHAPE#	dmax-Range(um)	re-Range(um)                  *
 *      ag2m        11      200 - 8864      65 - 1219                         *  
 *	    ag4m	    12	    200 - 10701	    65 - 1327	              	      *
 *      agc         13      200 - 12584     65 - 1124                         *
 *                                                                            * 
 *     Note: While the program will return values when dmax is out of         *
 *           range, the resultant values are probably wrong because           *
 *           they are EXTRAPOLATED. All negative values appear as -999.       *
 *           Alpha, arxz and aryz are not available (given as 999) for        *
 *           dmax smaller than ~760 um (dependent on frequency) as            *
 *           200 - 700 um sizes are 6-bullet rosette crystals from            *
 *           Liu's SCATDB.                                                    *
 *            	                                                              *	
 * Output:							              *
 *  cabs  - absorption cross section m^2  real/float		              *	
 *  csca  - scattering cross section m^2  real/float		              *
 *  cbsc  - backscattering cross section m^2 real/float		              *
 *  g     - asymmetry parameter	real/float		                      *
 *  re    - sphere-equivalent radius (assume density=0.916) real/float        *
 *  alpha - Haensch alpha statistic real/float				      *
 *  arxz  - aspect ratio calculated using a 2D image of X-Z axes real/float   *
 *  aryz  - aspect ratio calculated using a 2D image of Y-Z axes real/float   *
 *  iret  - return status	integer/int				      *
 *	   0	- 	success					              * 
 *     xxxx1(2)   - 	frequency higher (lower) than 183.31 (10.65) GHz      *
 *     xxx1x      -	temperature not equal to 263K	                      *
 *     xx1(2)xx  -	dmax larger (smaller) than that in database           *
 *                       for low frequency used in estimation                 * 
 *     x1(2)xxx  -	dmax larger (smaller) than that in database           *
 *                       for high frequency used in estimation                *
 *                                                                            *
 *     NOTE: Extrapolation will be used for 10>iret>0                         *
 *                                                                            *
 *     10000	- 	cannot find flake type       		              *
 *      	        No valid return values			              * 
 *     20000    -   cannot find scatdb_agdb.txt file                          *
 *								              *  
 * In&Out:							              *
 *  is_loaded - indicator whether dda_database is loaded to memory,           *
 *              assign 0 at the beginning of the program and Never            *
 *              change it again. integer/int                                  *
 *   NOTE:							              *
 *   The scatdb_agdb.txt file contains the data computed by DDA. It needs     *
 *   to be located in the directory indicated by the SCATDB_DATA              *
 *   environmental variable. If the variable is not set, the program          * 
 *   assumes the data file is in the current (run-time) directory.            *
 *   To set SCATDB_DATA, under csh/tcsh:                                      *
 *          setenv SCATDB_DATA (the-directory)			              *
 *   under sh/bash:						              *
 *           export SCATDB_DATA=(the-directory)                               *
 * ***************************************************************************/

/* Ver2.0  2012.2.24 */
/* Ver2.1  2012.3.23: Corrected linear_interp and is_loaded issues */
/* Ver2.2  2012.3.30: added scattering lines one standard deviation below and
                      above mean scattering line. NOTE: call has changed. */
/* Ver2.3  2012.7.18: Removed the high and low scattering lines; Added in 
                      13.6 GHz. NOTE: call has changed. */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#define	TRUE	1
#define FALSE	0

#define LSIZE 1390 //length of file
#define FSIZE 9   //number of frequencies
#define ASIZE 3   //number of flake types

  void scatdb_ag_(float*,float*,int*,float*,float*,float*,float*,float*,float*,float*,float*,float*,int*,int*);
  int scatdb_ag(float,float,int,float,float*,float*,float*,float*,float*,float*,float*,float*,int*);
  float linear_interp2(float,float,float,float,float);
  
/* wrapper for fortran interface */  
  
void scatdb_ag_(float *f,float *t,int *nshape,float *dmax,float *coabs,float *csca,float *cbsc,float *asym, \
             float *re, float *alpha,float *arxz,float *aryz,int *iret,int *is_loaded) 
{
	float f1, t1, dmax1;
	int n1;
	f1 = *f;
	t1 = *t;
	n1 = *nshape;
	dmax1 = *dmax;
	*iret = scatdb_ag(f1,t1,n1,dmax1,coabs,csca,cbsc,asym,re,alpha,arxz,aryz,is_loaded);
}  
 
/* linear interpolation */ 
float linear_interp2(float x,float x1,float x2,float y1,float y2) 
{
	float y;
    y=y1+(y2-y1)/(x2-x1)*(x-x1);
	return y;  
}

int scatdb_ag(float f,float t,int nshape,float dmax,float *coabs,float *csca,float *cbsc,float *asym, \
           float *re,float *alpha,float *arxz,float *aryz,int *is_loaded) 
{
  int i,k,iret=0,it1=-1,it2=-1,irl1=-1,irl2=-1,irh1=-1,irh2=-1,max_type=12,type_check=0,ntrial,flake_type,
      strial,scat_check=0;
  float if1=-1,if2=-1,dmaxp,lfldsi,lfhdsi,lfldre,lfhdre,lfldabs,lfhdabs,lfldsca,lfhdsca,lfldbsc,lfhdbsc,
        lfldg,lfhdg,lflda,lfhda,lfldarx,lfhdarx,lfldary,lfhdary,hfldsi,hfhdsi,hfldre,hfhdre,
        hfldabs,hfhdabs,hfldsca,hfhdsca,hfldbsc,hfhdbsc,hfldg,hfhdg,hflda,hfhda,hfldarx,hfhdarx,
        hfldary,hfhdary,lowsi,highsi,lowre,highre,re_1,coabs_1,csca_1,cbsc_1,asym_1,alpha_1,arxz_1,aryz_1;
  static int dtype[LSIZE],atype[ASIZE];
  static float freq[LSIZE],diam[LSIZE],reff[LSIZE],den[LSIZE],temp[LSIZE],
               extc[LSIZE],absc[LSIZE],scac[LSIZE],g[LSIZE],bscc[LSIZE],
	           alp[LSIZE],arx[LSIZE],ary[LSIZE];

  static float all_f[FSIZE] = {10.65,18.7,23.8,35.6,36.5,89,94,165.5,183.31};
  static float all_type[ASIZE] = {11,12,13};                               
  float x1,x2,y1,y2,a1,b1,c1,d1;
  
  ntrial=nshape;       
  
/* Currently only have 263K. Give a warning if user input not equal to 263k */

  if (t != 263.0)
   {
    return(iret=10);  
    exit(0);
   }

/* Check to see if correct flake type */
  for(k=0;k<ASIZE;++k)
  {
    if (nshape == all_type[k])
    {
     type_check=1;
    } 
  } 

  if (type_check == 0)
  {
     return(iret=10000);
     exit(0); 
  }
  

/*Checking the all_f array to see what frequencies the given one falls between */

  for (k=0;k<FSIZE-1;++k)
  {
    if ((f>=all_f[k]) && (f<=all_f[k+1])) 
     {
        if1=all_f[k];
        if2=all_f[k+1];
        break; 
     } 
  }
  
  if(if1 == -1) 
  {
		if(f<all_f[0]) 
        {
          if1=all_f[0];if2=all_f[1];iret += 2;
        }
		else 
        {
          if1=all_f[FSIZE-2];if2=all_f[FSIZE-1];iret += 1;
        }
  }
  
/* Reading in input file and storing values in an array */
  char scat_db_dir[180]={"."};
  const char scat_db_fn[]={"scatdb_agdb.txt"};
  FILE *fp;
  char *db_dir, wk[180];
	
  if(*is_loaded != TRUE) 
  {
	if((db_dir=getenv("SCATDB_DATA")))
	   strcpy(scat_db_dir,db_dir);
	   sprintf(wk,"%s/%s",scat_db_dir,scat_db_fn);
   
	if(!(fp=fopen(wk,"rb")))	
    {
			fprintf(stderr,"Cannot find scatdb_agdb.txt file. %s\n",wk);
			return(iret=20000);
    }  
  for(i=0;i<LSIZE;++i)
   {
    fscanf(fp,"%d %f %f %f %f %f %f %f %f %f %f %f \
               %f %f",&dtype[i],&freq[i],&temp[i],&diam[i], 
               &reff[i],&den[i],&extc[i],&absc[i],&scac[i], 
               &g[i],&bscc[i],&alp[i],&arx[i],&ary[i]);
   }
  
  fclose(fp); 
  *is_loaded = TRUE; 
  }
/* Checking what diameters desired value falls between for lower frequency */  

  for(k=0;k<LSIZE;++k)
  {
    if ((dmax>=diam[k]) && (dmax<=diam[k+1]) && (freq[k]==if1) && (dtype[k]==ntrial)) 
     {
        irl1=k;
        irl2=k+1;
        break; 
     } 
  }
  
	if(irl1 == -1) 
    {
      for(k=0;k<LSIZE;++k)
      {
       if ((freq[k]==if1) && (dtype[k]==ntrial))
       {   
		if((dmax<diam[k]) && (diam[k-1]>diam[k])) 
        {
          irl1=k;irl2=k+1;iret += 200;
        }
		else if((dmax>diam[k]) && (diam[k]>diam[k+1])) 
        {
          irl1=k-1;irl2=k;iret += 100;
        }
       }
      }
     }
     
/* Checking what diameters desired value falls between for higher frequency */
  
  for(k=0;k<LSIZE;++k)
  {
    if ((dmax>=diam[k]) && (dmax<=diam[k+1]) && (freq[k]==if2) && (dtype[k]==ntrial)) 
     {
        irh1=k;
        irh2=k+1;
        break; 
     } 
  }
  
	if(irh1 == -1) 
    {
      for(k=0;k<LSIZE;++k)
      {
       if ((freq[k]==if2) && (dtype[k]==ntrial))
       {   
		if((dmax<diam[k]) && (diam[k-1]>diam[k])) 
        {
          irh1=k;irh2=k+1;iret += 2000;
        }
		else if((dmax>diam[k]) && (diam[k]>diam[k+1])) 
        {
          irh1=k-1;irh2=k;iret += 1000;
        }
       }
      }
     }

        lfldsi=diam[irl1];
        lfhdsi=diam[irl2];
        lfldre=reff[irl1];
        lfhdre=reff[irl2];
        lfldabs=absc[irl1];
        lfhdabs=absc[irl2];
        lfldsca=scac[irl1];
        lfhdsca=scac[irl2];
        lfldbsc=bscc[irl1];
        lfhdbsc=bscc[irl2];
        lfldg=g[irl1];
        lfhdg=g[irl2];
        lflda=alp[irl1];
        lfhda=alp[irl2];
        lfldarx=arx[irl1];
        lfhdarx=arx[irl2];
        lfldary=ary[irl1];
        lfhdary=ary[irl2];

        hfldsi=diam[irh1];
        hfhdsi=diam[irh2];
        hfldre=reff[irh1];
        hfhdre=reff[irh2];
        hfldabs=absc[irh1];
        hfhdabs=absc[irh2];
        hfldsca=scac[irh1];
        hfhdsca=scac[irh2];
        hfldbsc=bscc[irh1];
        hfhdbsc=bscc[irh2];
        hfldg=g[irh1];
        hfhdg=g[irh2];
        hflda=alp[irh1];
        hfhda=alp[irh2];
        hfldarx=arx[irh1];
        hfhdarx=arx[irh2];
        hfldary=ary[irh1];
        hfhdary=ary[irh2];

/* sphere equavalent radius */
	x1=if1; x2=if2;
	y1=lfldre; y2=hfldre;
	a1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfhdre; y2=hfhdre;
	b1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfldsi; y2=hfldsi;
	lowsi=linear_interp2(f,x1,x2,y1,y2);
    y1=lfhdsi; y2=hfhdsi;
    highsi=linear_interp2(f,x1,x2,y1,y2);
	x1=lowsi; x2=highsi; y1=a1; y2=b1;
	re_1=linear_interp2(dmax,x1,x2,y1,y2);
	if (re_1 <= 0){*re=-999;}
    else{*re=re_1;}   
    
/* absorption */
	x1=if1; x2=if2;
	y1=lfldabs; y2=hfldabs;
	a1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfhdabs; y2=hfhdabs;
	b1=linear_interp2(f,x1,x2,y1,y2);	
	y1=lfldsi; y2=hfldsi;
	lowsi=linear_interp2(f,x1,x2,y1,y2);
    y1=lfhdsi; y2=hfhdsi;
    highsi=linear_interp2(f,x1,x2,y1,y2);
	x1=lowsi; x2=highsi; y1=a1; y2=b1;
	coabs_1=linear_interp2(dmax,x1,x2,y1,y2);
	if (coabs_1 <= 0){*coabs=-999;}
    else{*coabs=coabs_1;} 
	
/* scattering */
	x1=if1; x2=if2;
	y1=lfldsca; y2=hfldsca;
	a1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfhdsca; y2=hfhdsca;
	b1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfldsi; y2=hfldsi;
	lowsi=linear_interp2(f,x1,x2,y1,y2);
    y1=lfhdsi; y2=hfhdsi;
    highsi=linear_interp2(f,x1,x2,y1,y2);
	x1=lowsi; x2=highsi; y1=a1; y2=b1;
	csca_1=linear_interp2(dmax,x1,x2,y1,y2);
	if (csca_1 < 0){*csca=-999;}
    else{*csca=csca_1;}	
	
/* backscatter */
	x1=if1; x2=if2;
	y1=lfldbsc; y2=hfldbsc;
	a1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfhdbsc; y2=hfhdbsc;
	b1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfldsi; y2=hfldsi;
	lowsi=linear_interp2(f,x1,x2,y1,y2);
    y1=lfhdsi; y2=hfhdsi;
    highsi=linear_interp2(f,x1,x2,y1,y2);
	x1=lowsi; x2=highsi; y1=a1; y2=b1;
	cbsc_1=linear_interp2(dmax,x1,x2,y1,y2);
	if (cbsc_1 < 0){*cbsc=-999;}
    else{*cbsc=cbsc_1;}
    
/* asymmetry */
	x1=if1; x2=if2;
	y1=lfldg; y2=hfldg;
	a1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfhdg; y2=hfhdg;
	b1=linear_interp2(f,x1,x2,y1,y2);
	y1=lfldsi; y2=hfldsi; 	
	lowsi=linear_interp2(f,x1,x2,y1,y2);	
    y1=lfhdsi; y2=hfhdsi;
    highsi=linear_interp2(f,x1,x2,y1,y2);
	x1=lowsi; x2=highsi; y1=a1; y2=b1;
    asym_1=linear_interp2(dmax,x1,x2,y1,y2);
	if (asym_1 <= 0){*asym=-999;}
    else{*asym=asym_1;}
    
/* alpha */
    if ((lflda == 999) || (hflda == 999) || (lfhda == 999) ||(hfhda == 999))
    {
      *alpha=999;
    }
    else 
    { 
	  x1=if1; x2=if2;
	  y1=lflda; y2=hflda;
	  a1=linear_interp2(f,x1,x2,y1,y2);
	  y1=lfhda; y2=hfhda;
	  b1=linear_interp2(f,x1,x2,y1,y2);
	  y1=lfldsi; y2=hfldsi;
	  lowsi=linear_interp2(f,x1,x2,y1,y2);
      y1=lfhdsi; y2=hfhdsi;
      highsi=linear_interp2(f,x1,x2,y1,y2);
	  x1=lowsi; x2=highsi; y1=a1; y2=b1;
	  alpha_1=linear_interp2(dmax,x1,x2,y1,y2);
	  if (alpha_1 <= 0){*alpha=-999;}
      else{*alpha=alpha_1;}
    }	
             	
/* aspect ratio xz */
     if ((lfldarx == 999) || (hfldarx == 999) || (lfhdarx == 999) || (hfhdarx == 999))
     {
       *arxz=999;
     }
     else 
     {
	   x1=if1; x2=if2;
       y1=lfldarx; y2=hfldarx;
	   a1=linear_interp2(f,x1,x2,y1,y2);
	   y1=lfhdarx; y2=hfhdarx;
	   b1=linear_interp2(f,x1,x2,y1,y2);
	   y1=lfldsi; y2=hfldsi;
	   lowsi=linear_interp2(f,x1,x2,y1,y2);
       y1=lfhdsi; y2=hfhdsi;
       highsi=linear_interp2(f,x1,x2,y1,y2);
	   x1=lowsi; x2=highsi; y1=a1; y2=b1;
       arxz_1=linear_interp2(dmax,x1,x2,y1,y2);
	   if (arxz_1 <= 0){*arxz=-999;}
       else{*arxz=arxz_1;}
      }	  
    
/* aspect ratio yz */
    if ((lfldary == 999) || (hfldary == 999) || (lfhdary == 999) || (hfhdary == 999))
    {
      *aryz=999;
    }
    else 
    {
	  x1=if1; x2=if2;
	  y1=lfldary; y2=hfldary;
	  a1=linear_interp2(f,x1,x2,y1,y2);
	  y1=lfhdary; y2=hfhdary;
	  b1=linear_interp2(f,x1,x2,y1,y2);
	  y1=lfldsi; y2=hfldsi;
	  lowsi=linear_interp2(f,x1,x2,y1,y2);
      y1=lfhdsi; y2=hfhdsi;
      highsi=linear_interp2(f,x1,x2,y1,y2);
	  x1=lowsi; x2=highsi; y1=a1; y2=b1;
	  aryz_1=linear_interp2(dmax,x1,x2,y1,y2);
	  if (aryz_1 <= 0){*aryz=-999;}
      else{*aryz=aryz_1;}
	  *aryz=linear_interp2(dmax,x1,x2,y1,y2);
    }	
  
 
  return iret;

}
